/* This file has been prepared for Doxygen automatic documentation generation.*/
/*! \file *********************************************************************
 *
 * \brief This file shows how to implement a half duplex software driven uart.
 *
 * This single file shows how to implement a software uart using
 * Timer0 and external interrupt 0.
 *
 * Note that the RX_PIN must be the external interrupt 0 pin on
 * your AVR of choice. The TX_PIN can be chosen to be any suitable
 * pin. Note that this code is intended to run from an internal
 * 8 MHz clock source.
 *
 * \par Application note:
 *      AVR1907: Xplain Evaluation Board
 *
 * \par Documentation
 *      For comprehensive code documentation, supported compilers, compiler
 *      settings and supported devices see readme.html
 *
 * \author
 *      Atmel Corporation: http://www.atmel.com \n
 *      Support email: avr@atmel.com
 *
 * $Revision: 3137 $
 * $Date: 2010-01-18 09:21:33 +0100 (ma, 18 jan 2010) $  \n
 *
 * Copyright (c) 2010, Atmel Corporation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. The name of ATMEL may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ATMEL "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE EXPRESSLY AND
 * SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *****************************************************************************/

#include "sw_usart.h"

/*! \brief  External interrupt service routine.
 *
 *  The falling edge in the beginning of the start
 *  bit will trig this interrupt. The state will
 *  be changed to RECEIVE, and the timer interrupt
 *  will be set to trig one and a half bit period
 *  from the falling edge. At that instant the
 *  code should sample the first data bit.
 *
 *  \note  SW_USART_init(void) must be called in advance.
 */
ISR(INT0_vect)
{
	SwUsartRXState = RECEIVE;                     /* Change state */
	SW_USART_RX_DISABLE_EXTERNAL0_INTERRUPT();    /* Disable interrupt during the data bits. */
	
	SW_USART_RX_DISABLE_TIMER_INTERRUPT();        /* Disable timer to change its registers. */
	SW_USART_RX_DISABLE_PRESCALER();              /* Reset prescaler counter. */
	
	SW_USART_RX_TIMER = INTERRUPT_EXEC_CYCL;      /* Clear counter register. Include time to run interrupt rutine. */
	SW_USART_RX_COMPARE = SwUartTICKS2WAITONE;    /* Count one and a half period into the future. */
	SW_USART_RX_ENABLE_PRESCALER();               /* Start prescaler clock. */

	SwUartRXBitCount = 0;                         /* Clear received bit counter. */
	SW_USART_RX_CLEAR_TIMER_INTERRUPT();          /* Clear interrupt bits */
	SW_USART_RX_ENABLE_TIMER_INTERRUPT();         /* Enable timer0 interrupt on again */
	
}


/*! \brief  Timer0 interrupt service routine.
 *
 *  Timer0 will ensure that bits are
 *  read at the correct instants in time.
 *
 *  If state should be something else, the
 *  variable is set to IDLE. IDLE is regarded
 *  as a safe state/mode.
 *
 *  \note  SW_USART_init(void) must be called in advance.
 */
ISR(SW_USART_RX_COMP_VECT)
{
	/* Get state of RX pin. */
	uint8_t rx_pinstate = SW_USART_GET_RX_PIN();
	
	/* If 8 bit have not been received, capture and add bit to input byte.*/
	if( SwUartRXBitCount < 8 ) {

		/* Rightshift due to receiving LSB first.*/
		SwUartRXData = (SwUartRXData>>1);

	        /* If a logical 1 is read on the pin, add this to the MSB
		 * of the input byte.
		 */
		if(rx_pinstate != 0 ){
			SwUartRXData |= 0x80;
		}
		
		SwUartRXBitCount++;
	}
	
	/* Complete byte received, send stop byte.
	 * Store the received byte in the buffer.
	 * Disable this interrupt for bit receiving and enable
	 * pin change interrupt for detecting start bit.
	 */
	else{
		/* If STOP bit is detected, add byte to buffer. If STOP was not
		 * detected, discard the byte and reset receiver.
		 */
		if(rx_pinstate != 0 ){
			SW_USART_RX_Buffer_PutByte(SwUartRXData);
		}
		SwUsartRXState = IDLE;
		SW_USART_RX_DISABLE_TIMER_INTERRUPT();
		SW_USART_RX_DISABLE_PRESCALER();
		SW_USART_EXT_IFR |= (1 << INTF0);
		SW_USART_RX_ENABLE_EXTERNAL0_INTERRUPT();
	}
}

/*! \brief Timer2 interrupt service routine.
 *
 *  Timer2 will ensure that bits are written
 *  at the correct instants in time.
 *
 *  If state should be something else, the
 *  variable is set to IDLE. IDLE is regarded
 *  as a safe state/mode.
 *
 *  \note  SW_USART_init(void) must be called in advance.
 */
ISR(SW_USART_TX_COMP_VECT)
{
	/* Start bit has already been sent, process the byte and stop bit. */
	switch (SwUsartTXState) {

	    /* Transmit byte in TX buffer. */
	    case TRANSMIT:
		
		/* If the complete byte is not transmitted, send next bit.*/
		if( SwUartTXBitCount < 8 ) {
			if( SwUartTXData & 0x01 ){
				SW_USART_SET_TX_PIN();
			}else{
				SW_USART_CLR_TX_PIN();
			}
			
			/* Rightshift the tx byte to be ready for the next bit transmit. */
			SwUartTXData = SwUartTXData >> 1;
			SwUartTXBitCount++;
		}
		
		/* Send stop bit if complete byte have been sent. */
		else {
			SW_USART_SET_TX_PIN();
			SwUsartTXState = TRANSMIT_STOP_BIT;
		}
		break;
		
	    /* Stop bit has been sent, go to idle and disable this interrupt. */
	    case TRANSMIT_STOP_BIT:
		SW_USART_TX_DISABLE_TIMER_INTERRUPT();
		SwUsartTXState = IDLE;
		break;
		
	    /* Unknown state. This indicates an error, but this should never occur.
	     * Set state to IDLE to be able to continue operation.
	     */
	    default:
		SW_USART_TX_DISABLE_TIMER_INTERRUPT();
		SwUsartTXState = IDLE;
	}
}


/*! \brief  Function to initialize the software UART.
 *
 *  This function will set up pins to transmit and
 *  receive on. Control of Timer0 and External interrupt 0.
 *
 */
void SW_USART_init( void )
{
	/* Setup the TRX PORT for the Soft UART.
	 * RX_PIN is input, tri-stated.
	 * TX_PIN is output, set high to IDLE state.
	 */
	SW_USART_TRX_PORT |= ( 1 << SW_USART_RX_PIN );
	SW_USART_TRX_DDR |= ( 1 << SW_USART_TX_PIN );
	SW_USART_SET_TX_PIN();
	
	/* Disable all TX and RX interrupts. */
	SW_USART_RX_DISABLE_TIMER_INTERRUPT();
	SW_USART_TX_DISABLE_TIMER_INTERRUPT();
	
	/* Initialize the RX Timer to CTC mode.
	 * This will clear the timer on compare match, and the counter start
	 * at bottom again. The compare value is equal to on bit length.
	 */
	SW_USART_RX_PRESCALE = 0x00;
	SW_USART_RX_CTRL = 0x02;
	SW_USART_RX_COMPARE = SwUartTICKS2WAITONE;

	/* Initialize the TX Timer to CTC mode.
	 * This will clear the timer on compare match, and the counter start
	 * at bottom again. The compare value is equal to on bit length.
	 */
	SW_USART_TX_PRESCALE = 0x00;
	SW_USART_TX_CTRL = 0x02;
	SW_USART_TX_COMPARE = SwUartTICKS2WAITONE;
	
	/* Initialize the RX external interrupt for sensing the start bit.
	 * This will trigger on the falling edge of the IO pin.
	 * Clear any pending interrupts and enable the interrupt.
	 */
	SW_USART_EXT_ICR |= ( 1 << ISC01 );
	SW_USART_EXT_IFR |= (1 << INTF0 );
	SW_USART_RX_ENABLE_EXTERNAL0_INTERRUPT();
	
	/* Set TRX states initially to IDLE. */
	SwUsartRXState = IDLE;
	SwUsartTXState = IDLE;
}


/*! \brief  Send a unsigned char.
 *
 *  This function sends a unsigned char on the TX_PIN
 *  using the timer2 isr.
 *
 *  \note  SW_USART_init(void) must be called in advance.
 *
 *  \param  c    char to transmit.
 *
 *  \retval void
 */
void SW_USART_PutChar( const uint8_t c )
{
	/* Wait until the TX is not busy or the communication will fail */
	while( SwUsartTXState != IDLE ){
	}
	
	/* Disable the clock to the timer and set the compare value to one bit length. */
	SW_USART_TX_DISABLE_PRESCALER();
	SW_USART_TX_COMPARE = SwUartTICKS2WAITONE;

	/* Change state to indicate that the TX is busy. */
	SwUsartTXState = TRANSMIT;

	/* Set intial bitcount value and copy byte into the TX buffer. */
	SwUartTXData = c;
	SwUartTXBitCount = 0;
	
	/* Clear the timer counter value and clear any pending interrupts. */
	SW_USART_TX_TIMER = 0;
	SW_USART_TX_CLEAR_TIMER_INTERRUPT();
	
	/* Start the timer, set pin value to transmit start bit and enable interrupt. */
	SW_USART_TX_ENABLE_PRESCALER();
	SW_USART_CLR_TX_PIN();
	SW_USART_TX_ENABLE_TIMER_INTERRUPT();
}


/*! \brief Function to check if data is pending in the buffer.
 *
 *  \retval true if data is pending, false otherwise
 */
uint8_t SW_USART_Test_Hit(void)
{
	return (SW_USART_RX_Num > 0);
}


/*! \brief Function to check if transmitter is busy.
 *
 *  \retval true if ready, false otherwise.
 */
uint8_t SW_USART_Tx_Ready(void)
{
	return ( SwUsartTXState == IDLE );
}


/*! \brief Function to put byte in RX buffer
 *
 *  \note This function is called from an interrupt and might change the pointers
 *        while the SW_USART_RX_Buffer_GetByte(void) function access the same values.
 *
 *  \param rx_byte to put into buffer.
 */
void SW_USART_RX_Buffer_PutByte(uint8_t rx_byte)
{
	/* Check if buffer is full. The byte will be lost if the buffer is full.*/
	if(SW_USART_RX_Num < SW_USART_BUFFER_SIZE ){
		SW_USART_RX_Buffer[SW_USART_RX_Head] = rx_byte;
		
		/* Increase the number of bytes in buffer and the pointer. */
		SW_USART_RX_Num++;
		SW_USART_RX_Head++;
		
		/* Check if we should wrap around. */
		if(SW_USART_RX_Head >= SW_USART_BUFFER_SIZE){
			SW_USART_RX_Head = 0;
		}
	}	
}


/*! \brief Get byte from the RX buffer.
 *
 *  \note The pointers might be changed in interrupts, and it's important to disable
 *        the global interrupt while manipulating the values that might be changed
 *        in the interrupt.
 *
 *  \retval Byte from buffer
 */
uint8_t SW_USART_RX_Buffer_GetByte(void)
{
	uint8_t ret_byte = 0;
	
	/* Check if there is any bytes in the buffer. */
	if(SW_USART_RX_Num > 0){
		ret_byte = SW_USART_RX_Buffer[SW_USART_RX_Tail];
	
		/* Enter a critical region as the value might be changed in an interrupt. */
		AVR_ENTER_CRITICAL_REGION();
		
		/* Decrease number of bytes left in buffer. */
		SW_USART_RX_Num--;
		
		/* Leave the critical region and restore SREG register. */
		AVR_LEAVE_CRITICAL_REGION();
		
		/* Increase the tail pointer and wrap around if necessary. */
		SW_USART_RX_Tail++;
		if(SW_USART_RX_Tail >= SW_USART_BUFFER_SIZE){
			SW_USART_RX_Tail = 0;
		}
	}
	return ret_byte;
}


/*! \brief Set new baudrate.
 *
 *  \note Change do not work yet, do not use.
 *
 *  \param new_baud The new baudrate to set.
 */
void SW_USART_Set_Baudrate(uint32_t new_baud)
{
//	SwUartTICKS2WAITONE = (uint8_t) ((1000000/new_baud)-1);
}
