/* This file has been prepared for Doxygen automatic documentation generation.*/
/*! \file *********************************************************************
 *
 * \brief This file shows how to implement a half duplex software driven uart.
 *
 * This single file shows how to implement a software uart using
 * Timer0 and external interrupt 0.
 *
 * Note that the RX_PIN must be the external interrupt 0 pin on
 * your AVR of choice. The TX_PIN can be chosen to be any suitable
 * pin. Note that this code is intended to run from an internal
 * 8 MHz clock source.
 *
 * \par Application note:
 *      AVR1907: Xplain Evaluation Board
 *
 * \par Documentation
 *      For comprehensive code documentation, supported compilers, compiler
 *      settings and supported devices see readme.html
 *
 * \author
 *      Atmel Corporation: http://www.atmel.com \n
 *      Support email: avr@atmel.com
 *
 * $Revision: 3122 $
 * $Date: 2010-01-13 13:26:22 +0100 (on, 13 jan 2010) $  \n
 *
 * Copyright (c) 2010, Atmel Corporation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. The name of ATMEL may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ATMEL "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE EXPRESSLY AND
 * SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *****************************************************************************/

#ifndef _SW_USART_MODULE_H_
#define _SW_USART_MODULE_H_

#include "config.h"

/****************************** DEFINES ***************************************/

/*! \brief  Timer prescaler enumeration.
 */
typedef enum TIMER_PRESCALER_enum {
	TIMER_PRESCALER_OFF_gc         = (0x00), /*!< Timer prescaler off. Timer stopped. */
	TIMER_PRESCALER_1_gc           = (0x01), /*!< Timer runs at CPU speed. */
	TIMER_PRESCALER_8_gc           = (0x02), /*!< Timer runs at CPU/8 speed. */
	TIMER_PRESCALER_64_gc          = (0x03), /*!< Timer runs at CPU/64 speed. */
	TIMER_PRESCALER_256_gc         = (0x04), /*!< Timer runs at CPU/256 speed. */
	TIMER_PRESCALER_1024_gc        = (0x05), /*!< Timer runs at CPU/1024 speed. */
	TIMER_PRESCALER_EXT_FALLING_gc = (0x06), /*!< Timer runs of external clock. Falling edge triggered. */
	TIMER_PRESCALER_EXT_RISING_gc  = (0x07), /*!< Timer runs of external clock. Rising edge triggered. */
} TIMER_PRESCALER_t;


/*! \brief  Type defined enumeration holding software UART's state.
 */
typedef enum
{
    IDLE,                 /*!< Idle state, both transmit and receive possible.*/
    TRANSMIT,             /*!< Transmitting byte. */
    TRANSMIT_STOP_BIT,    /*!< Transmitting stop bit.*/
    RECEIVE,              /*!< Receiving byte.*/
    DATA_PENDING          /*!< Byte received and ready to read.*/
}AsynchronousStates_t;

#define SW_USART_BUFFER_SIZE 32  /*!< Software buffer size */
#define SW_USART_BAUD 9600       /*!< Desired baudrate. In current implementation only 4800 and 9600 baud is supported. */

/*! \brief Wait one bit period. */
#define SW_USART_TICKS2WAITONE (uint8_t) ((1000000 / SW_USART_BAUD) - 1)

/*! \brief Cycles to execute interrupt rutine from interrupt.*/
#define INTERRUPT_EXEC_CYCL    (0xFF - 10)

/* Some IO, timer and interrupt specific defines. */

#define SW_USART_RX_CTRL        TCCR0A             /*!< Timer/Counter Control Register */
#define SW_USART_RX_PRESCALE    TCCR0B             /*!< Timer/Counter Control (Prescaler) Register */
#define SW_USART_RX_COMPARE     OCR0A              /*!< Output Compare Register */
#define SW_USART_RX_TIMER       TCNT0              /*!< Timer Counter register */
#define SW_USART_EXT_IFR        EIFR               /*!< External Interrupt Flag Register */
#define SW_USART_EXT_ICR        EICRA              /*!< External Interrupt Control Register */
#define SW_USART_RX_COMP_VECT   TIMER0_COMPA_vect  /*!< Timer Compare Interrupt Vector */

#define SW_USART_TX_CTRL        TCCR2A             /*!< Timer/Counter Control Register */
#define SW_USART_TX_PRESCALE    TCCR2B             /*!< Timer/Counter Control (Prescaler) Register */
#define SW_USART_TX_COMPARE     OCR2A              /*!< Output Compare Register */
#define SW_USART_TX_TIMER       TCNT2              /*!< Timer Counter register */
#define SW_USART_TX_COMP_VECT   TIMER2_COMPA_vect  /*!< Timer Compare Interrupt Vector */

#define SW_USART_TX_PIN    PD1                     /*!< Transmit data pin */
#define SW_USART_RX_PIN    PD0                     /*!< Receive data pin, must be INT0 */
#define SW_USART_TRX_DDR   DDRD                    /*!< TRX Data direction register */
#define SW_USART_TRX_PORT  PORTD                   /*!< TRX Port register */
#define SW_USART_TRX_PIN   PIND                    /*!< TRX Pin register */


/* Select prescaler so the timer runs at 1MHz. */
#ifdef FOSC
#	if (FOSC == 8000)
#		define SW_USART_PRESCALER TIMER_PRESCALER_8_gc
#	elif (FOSC == 1000)
#		define SW_USART_PRESCALER TIMER_PRESCALER_1_gc
#	else
#		error FOSC must be 1000 or 8000.
#	endif
#else
#	error FOSC must be defined.
#endif /* FOSC */

/****************************** MACROS ****************************************/

#define SW_USART_RX_ENABLE_TIMER_INTERRUPT()       ( TIMSK0 |= ( 1<< OCIE0A ) )
#define SW_USART_RX_DISABLE_TIMER_INTERRUPT()      ( TIMSK0 &= ~( 1<< OCIE0A ) )
#define SW_USART_RX_CLEAR_TIMER_INTERRUPT()        ( TIFR0 |= ((1 << OCF0A) ) )
#define SW_USART_RX_ENABLE_EXTERNAL0_INTERRUPT()   ( EIMSK |= ( 1<< INT0 ) )
#define SW_USART_RX_DISABLE_EXTERNAL0_INTERRUPT()  ( EIMSK &= ~( 1<< INT0 ) )
#define SW_USART_RX_ENABLE_PRESCALER()             ( SW_USART_RX_PRESCALE = SW_USART_PRESCALER )
#define SW_USART_RX_DISABLE_PRESCALER()            ( SW_USART_RX_PRESCALE = 0x00)

#define SW_USART_TX_ENABLE_TIMER_INTERRUPT()       ( TIMSK2 |= ( 1<< OCIE2A ) )
#define SW_USART_TX_DISABLE_TIMER_INTERRUPT()      ( TIMSK2 &= ~( 1<< OCIE2A ) )
#define SW_USART_TX_CLEAR_TIMER_INTERRUPT()        ( TIFR2 |= ((1 << OCF2A) ) )
#define SW_USART_TX_ENABLE_PRESCALER()             ( SW_USART_TX_PRESCALE = SW_USART_PRESCALER )
#define SW_USART_TX_DISABLE_PRESCALER()            ( SW_USART_TX_PRESCALE = 0x00)


#define SW_USART_SET_TX_PIN()  ( SW_USART_TRX_PORT |= ( 1 << SW_USART_TX_PIN ) )
#define SW_USART_CLR_TX_PIN()  ( SW_USART_TRX_PORT &= ~( 1 << SW_USART_TX_PIN ) )
#define SW_USART_GET_RX_PIN()  ( SW_USART_TRX_PIN & ( 1 << SW_USART_RX_PIN ) )


/************************* VARIABLES (LOCAL) **********************************/

static volatile AsynchronousStates_t SwUsartRXState;     /*!< Holds the state of the UART. */
static volatile AsynchronousStates_t SwUsartTXState;     /*!< Holds the state of the UART. */
static volatile uint8_t SwUartTXData;                    /*!< Data to be transmitted. */
static volatile uint8_t SwUartTXBitCount;                /*!< TX bit counter. */
static volatile uint8_t SwUartRXData;                    /*!< Storage for received bits. */
static volatile uint8_t SwUartRXBitCount;                /*!< RX bit counter. */
static volatile uint8_t SwUartTICKS2WAITONE = SW_USART_TICKS2WAITONE;

static volatile uint8_t SW_USART_RX_Buffer[SW_USART_BUFFER_SIZE];
static volatile uint8_t SW_USART_RX_Head = 0;
static volatile uint8_t SW_USART_RX_Tail = 0;
static volatile uint8_t SW_USART_RX_Num = 0;


/************************* PROTOTYPES (GLOBAL) ********************************/

void SW_USART_PutChar( const uint8_t c );
void SW_USART_RX_Buffer_PutByte(uint8_t);
void SW_USART_init( void );
void SW_USART_Set_Baudrate(uint32_t);
uint8_t SW_USART_RX_Buffer_GetByte(void);
uint8_t SW_USART_Tx_Ready(void);
uint8_t SW_USART_Test_Hit(void);

#endif /* _SW_USART_MODULE_H_ */