/* This file has been prepared for Doxygen automatic documentation generation.*/
/*! \file *********************************************************************
 *
 * \brief This file controls the UART USB functions.
 *
 * \par Application note:
 *      AVR1907: Xplain Evaluation Board
 *
 * \par Documentation
 *      For comprehensive code documentation, supported compilers, compiler
 *      settings and supported devices see readme.html
 *
 * \author
 *      Atmel Corporation: http://www.atmel.com \n
 *      Support email: avr@atmel.com
 *
 * $Revision: 3122 $
 * $Date: 2010-01-13 13:26:22 +0100 (on, 13 jan 2010) $  \n
 *
 * Copyright (c) 2010, Atmel Corporation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. The name of ATMEL may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ATMEL "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE EXPRESSLY AND
 * SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *****************************************************************************/

#include "config.h"
#include "usb_drv.h"
#include "usb_descriptors.h"
#include "uart_usb_lib.h"

/* Local counters for buffer. */
uint8_t tx_counter;
uint8_t rx_counter;

/*! \brief Initializes the uart_usb library
 */
void uart_usb_init(void)
{
	tx_counter = 0;
	rx_counter = 0;
}

/*! \brief This function checks if a character has been received on the USB bus.
 *
 *  \return bit (true if a byte is ready to be read)
 */
uint8_t uart_usb_test_hit(void)
{
	if (!rx_counter)
	{
		Usb_select_endpoint(RX_EP);
		if (Is_usb_receive_out())
		{
			
#ifdef __ICCAVR__
#pragma diag_suppress=Pa082 /* Suppress warning for undefined order of volatile access. */
#endif
			rx_counter = Usb_byte_counter();
#ifdef __ICCAVR__
#pragma diag_default=Pa082 /* Back to default.*/
#endif

			if (!rx_counter)
			{
				Usb_ack_receive_out();
			}
		}
	}
	return (rx_counter!=0);
}


/*! \brief This function reads one byte from the USB bus
 *
 *  If one byte is present in the USB fifo, this byte is returned. If no data
 *  is present in the USB fifo, this function waits for USB data.
 *
 *  \return byte received
 */
uint8_t uart_usb_getchar(void)
{
	register uint8_t data_rx;
	
	Usb_select_endpoint(RX_EP);
	if (!rx_counter){
		while (!uart_usb_test_hit());
	}
	
	data_rx = Usb_read_byte();
	rx_counter--;
	
	if (!rx_counter){
		Usb_ack_receive_out();
	}
	
	return data_rx;
}

/*! \brief This function checks if the USB emission buffer is ready to accept at
 *         at least 1 byte
 *
 *  \return TRUE if the firmware can write a new byte to transmit.
 */
uint8_t uart_usb_tx_ready(void)
{
	if (!Is_usb_write_enabled())
	{
		return false;
	}
	return true;
}

/*! \brief This function fills the USB transmit buffer with the new data. This buffer
 *         is sent if complete. To flush this buffer before waiting full, launch
 *         the uart_usb_flush() function.
 *
 *  \param data_to_send
 *
 *  \return
 */
int uart_usb_putchar(int data_to_send)
{
  	Usb_select_endpoint(TX_EP);
  	while(!uart_usb_tx_ready()); /* Wait Endpoint ready */
	Usb_write_byte(data_to_send);
	tx_counter++;
  	if(!Is_usb_write_enabled()) /* If Endpoint full -> flush */
  	{
		uart_usb_flush();
  	}
	return data_to_send;
}


/*! \brief This function sends the data stored in the USB transmit buffer.
 *
 *  This function does nothing if there is no data in the buffer.
 */
void uart_usb_flush (void)
{
	Usb_select_endpoint(TX_EP);
	Usb_send_in();
	tx_counter = 0;
	
}
