/* This file has been prepared for Doxygen automatic documentation generation.*/
/*! \file *********************************************************************
 *
 * \brief This file contains the usb parameters that uniquely identify the
 *        CDC application through descriptor tables.
 *
 * \par Application note:
 *      AVR1907: Xplain Evaluation Board
 *
 * \par Documentation
 *      For comprehensive code documentation, supported compilers, compiler
 *      settings and supported devices see readme.html
 *
 * \author
 *      Atmel Corporation: http://www.atmel.com \n
 *      Support email: avr@atmel.com
 *
 * $Revision: 3122 $
 * $Date: 2010-01-13 13:26:22 +0100 (on, 13 jan 2010) $  \n
 *
 * Copyright (c) 2010, Atmel Corporation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. The name of ATMEL may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ATMEL "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE EXPRESSLY AND
 * SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *****************************************************************************/


#ifndef _USB_USERCONFIG_H_
#define _USB_USERCONFIG_H_

#include "config.h"
#include "usb_standard_request.h"
#include "conf_usb.h"

/****************************** MACROS ****************************************/

#define Usb_get_dev_desc_pointer()        (&(usb_dev_desc.bLength))
#define Usb_get_dev_desc_length()         (sizeof (usb_dev_desc))
#define Usb_get_conf_desc_pointer()       (&(usb_conf_desc.cfg.bLength))
#define Usb_get_conf_desc_length()        (sizeof (usb_conf_desc))

/****************************** DEFINES ***************************************/

/* USB Device descriptor */
#define USB_SPECIFICATION     0x0200
#define DEVICE_CLASS          0x02   /*!< CDC class */
#define DEVICE_SUB_CLASS      0x00   /*!< Each configuration has its own sub-class. */
#define DEVICE_PROTOCOL       0x00   /*!< Each configuration has its own protocol. */
#define EP_CONTROL_LENGTH     64
#define VENDOR_ID             0x03EB /*!< Atmel vendor ID = 03EBh */
#define PRODUCT_ID            0x210D /*!< Xplain Product ID = 210Dh */
#define RELEASE_NUMBER        0x0102
#define LANG_INDEX            0x00
#define MAN_INDEX             0x01
#define PROD_INDEX            0x02
#define SN_INDEX              0x03
#define IF0_INDEX             0x04
#define NB_CONFIGURATION      1


/* CDC CONFIGURATION */
#define NB_INTERFACE       2
#define CONF_NB            1
#define CONF_INDEX         0
#define CONF_ATTRIBUTES    USB_CONFIG_BUSPOWERED
#define MAX_POWER          250        /* 500 mA given in 2mA steps*/

/* Interface 0 descriptor */
#define INTERFACE0_NB        0
#define ALTERNATE0           0
#define NB_ENDPOINT0         1
#define INTERFACE0_CLASS     0x02    /* CDC ACM Com */
#define INTERFACE0_SUB_CLASS 0x02
#define INTERFACE0_PROTOCOL  0x01
#define INTERFACE0_INDEX     IF0_INDEX

/* Interface 1 descriptor */
#define INTERFACE1_NB        1
#define ALTERNATE1           0
#define NB_ENDPOINT1         2
#define INTERFACE1_CLASS     0x0A    /* CDC ACM Data */
#define INTERFACE1_SUB_CLASS 0
#define INTERFACE1_PROTOCOL  0
#define INTERFACE1_INDEX     IF0_INDEX

/* USB Endpoint 3 descriptor */
#define TX_EP_SIZE          0x20
#define ENDPOINT_NB_3       EP_DIR_IN | INT_EP /*!< Interrupt IN */
#define EP_ATTRIBUTES_3     EP_ATTR_INTERUPT
#define EP_SIZE_3           TX_EP_SIZE
#define EP_INTERVAL_3       0xFF

/* USB Endpoint 1 descriptor */
#define ENDPOINT_NB_1       EP_DIR_IN | TX_EP
#define EP_ATTRIBUTES_1     EP_ATTR_BULK
#define EP_SIZE_1           0x20
#define EP_INTERVAL_1       0x00

/* USB Endpoint 2 descriptor */
#define ENDPOINT_NB_2       EP_DIR_OUT | RX_EP
#define EP_ATTRIBUTES_2     EP_ATTR_BULK
#define EP_SIZE_2           0x20
#define EP_INTERVAL_2       0x00


#define DEVICE_STATUS         0x00
#define INTERFACE_STATUS      0x00



/* Strings for indexing */
#define USB_MN_LENGTH         5
#define USB_MANUFACTURER_NAME {'A','T','M','E','L'}

#define USB_PN_LENGTH         18
#define USB_PRODUCT_NAME      {'X','p','l','a','i','n',' ','U','S','B',' ','G','a','t','e','w','a','y'}

#define USB_SN_LENGTH         8
#define USB_SERIAL_NUMBER     {'0','0','0','0','0','0','0','0'}

#define USB_IF0_LENGTH         11
#define USB_IF0_DESCRIPT      {'S','e','r','i','a','l',' ','P','o','r','t'}

#define LANGUAGE_ID           0x0409

/*! Usb Request */
typedef struct {
	uint8_t      bmRequestType;        /*!< Characteristics of the request */
	uint8_t      bRequest;             /*!< Specific request */
	uint16_t     wValue;               /*!< field that varies according to request */
	uint16_t     wIndex;               /*!< field that varies according to request */
	uint16_t     wLength;              /*!< Number of bytes to transfer if Data */
}  S_UsbRequest;


/*! Usb Device Descriptor */
typedef struct {
	uint8_t      bLength;              /*!< Size of this descriptor in bytes */
	uint8_t      bDescriptorType;      /*!< DEVICE descriptor type */
	uint16_t     bscUSB;               /*!< Binay Coded Decimal Spec. release */
	uint8_t      bDeviceClass;         /*!< Class code assigned by the USB */
	uint8_t      bDeviceSubClass;      /*!< Sub-class code assigned by the USB */
	uint8_t      bDeviceProtocol;      /*!< Protocol code assigned by the USB */
	uint8_t      bMaxPacketSize0;      /*!< Max packet size for EP0 */
	uint16_t     idVendor;             /*!< Vendor ID. ATMEL = 0x03EB */
	uint16_t     idProduct;            /*!< Product ID assigned by the manufacturer */
	uint16_t     bcdDevice;            /*!< Device release number */
	uint8_t      iManufacturer;        /*!< Index of manu. string descriptor */
	uint8_t      iProduct;             /*!< Index of prod. string descriptor */
	uint8_t      iSerialNumber;        /*!< Index of S.N.  string descriptor */
	uint8_t      bNumConfigurations;   /*!< Number of possible configurations */
}  S_usb_device_descriptor;


/*! Usb Configuration Descriptor */
typedef struct {
	uint8_t      bLength;              /*!< size of this descriptor in bytes */
	uint8_t      bDescriptorType;      /*!< CONFIGURATION descriptor type */
	uint16_t     wTotalLength;         /*!< total length of data returned */
	uint8_t      bNumInterfaces;       /*!< number of interfaces for this conf. */
	uint8_t      bConfigurationValue;  /*!< value for SetConfiguration resquest */
	uint8_t      iConfiguration;       /*!< index of string descriptor */
	uint8_t      bmAttibutes;          /*!< Configuration characteristics */
	uint8_t      MaxPower;             /*!< maximum power consumption */
}  S_usb_configuration_descriptor;

/*! Usb Interface Association Descriptor */
typedef struct {
	uint8_t      bLength;               /*!< size of this descriptor in bytes */
	uint8_t      bDescriptorType;       /*!< INTERFACE descriptor type */
	uint8_t      bFirstInterface;       /*!< Number of interface */
	uint8_t      bInterfaceCount;       /*!< value to select alternate setting */
	uint8_t      bFunctionClass;        /*!< Class code assigned by the USB */
	uint8_t      bFunctionSubClass;     /*!< Sub-class code assigned by the USB */
	uint8_t      bFunctionProtocol;     /*!< Protocol code assigned by the USB */
	uint8_t      iFunction;             /*!< Index of string descriptor */
}  S_usb_interface_association_descriptor;

/*! Usb Interface Descriptor */
typedef struct {
	uint8_t      bLength;               /*!< size of this descriptor in bytes */
	uint8_t      bDescriptorType;       /*!< INTERFACE descriptor type */
	uint8_t      bInterfaceNumber;      /*!< Number of interface */
	uint8_t      bAlternateSetting;     /*!< value to select alternate setting */
	uint8_t      bNumEndpoints;         /*!< Number of EP except EP 0 */
	uint8_t      bInterfaceClass;       /*!< Class code assigned by the USB */
	uint8_t      bInterfaceSubClass;    /*!< Sub-class code assigned by the USB */
	uint8_t      bInterfaceProtocol;    /*!< Protocol code assigned by the USB */
	uint8_t      iInterface;            /*!< Index of string descriptor */
}  S_usb_interface_descriptor;


/*! Usb Endpoint Descriptor */
typedef struct {
	uint8_t      bLength;               /*!< Size of this descriptor in bytes */
	uint8_t      bDescriptorType;       /*!< ENDPOINT descriptor type */
	uint8_t      bEndpointAddress;      /*!< Address of the endpoint */
	uint8_t      bmAttributes;          /*!< Endpoint's attributes */
	uint16_t     wMaxPacketSize;        /*!< Maximum packet size for this EP */
	uint8_t      bInterval;             /*!< Interval for polling EP in ms */
} S_usb_endpoint_descriptor;


/*! Usb Device Qualifier Descriptor */
typedef struct {
	uint8_t      bLength;               /*!< Size of this descriptor in bytes */
	uint8_t      bDescriptorType;       /*!< Device Qualifier descriptor type */
	uint16_t     bscUSB;                /*!< Binay Coded Decimal Spec. release */
	uint8_t      bDeviceClass;          /*!< Class code assigned by the USB */
	uint8_t      bDeviceSubClass;       /*!< Sub-class code assigned by the USB */
	uint8_t      bDeviceProtocol;       /*!< Protocol code assigned by the USB */
	uint8_t      bMaxPacketSize0;       /*!< Max packet size for EP0 */
	uint8_t      bNumConfigurations;    /*!< Number of possible configurations */
	uint8_t      bReserved;             /*!< Reserved for future use, must be zero */
}  S_usb_device_qualifier_descriptor;


/*! Usb Language Descriptor */
typedef struct {
	uint8_t      bLength;               /*!< size of this descriptor in bytes */
	uint8_t      bDescriptorType;       /*!< STRING descriptor type */
	uint16_t     wlangid;               /*!< language id */
}  S_usb_language_id;


/*! Usb Manufacturer Descriptor */
typedef struct {
	uint8_t  bLength;                   /*!< size of this descriptor in bytes */
	uint8_t  bDescriptorType;           /*!< STRING descriptor type */
	uint16_t wstring[USB_MN_LENGTH];    /*!< unicode characters */
} S_usb_manufacturer_string_descriptor;


/*! Usb Product Descriptor */
typedef struct {
	uint8_t  bLength;                   /*!< size of this descriptor in bytes */
	uint8_t  bDescriptorType;           /*!< STRING descriptor type */
	uint16_t wstring[USB_PN_LENGTH];    /*!< unicode characters */
} S_usb_product_string_descriptor;


/*! Usb Serial Number Descriptor */
typedef struct {
	uint8_t  bLength;                   /*!< size of this descriptor in bytes */
	uint8_t  bDescriptorType;           /*!< STRING descriptor type */
	uint16_t wstring[USB_SN_LENGTH];    /*!< unicode characters */
} S_usb_serial_number;


/*! Usb Interface 0 Descriptor */
typedef struct {
	uint8_t  bLength;                   /*!< size of this descriptor in bytes */
	uint8_t  bDescriptorType;           /*!< STRING descriptor type */
	uint16_t wstring[USB_IF0_LENGTH];   /*!< unicode characters */
} S_usb_if0_descript;


/*! Usb CDC Class Descriptor */
typedef struct {
	S_usb_configuration_descriptor cfg;
	S_usb_interface_descriptor     ifc0;
	uint8_t                        CS_INTERFACE[19];
	S_usb_endpoint_descriptor      ep3;
	S_usb_interface_descriptor     ifc1;
	S_usb_endpoint_descriptor      ep1;
	S_usb_endpoint_descriptor      ep2;
} S_usb_user_configuration_descriptor;


#endif

