/* This file has been prepared for Doxygen automatic documentation generation.*/
/*! \file *********************************************************************
 *
 * \brief This file manages the USB device controller.
 *
 * The USB task checks the income of new requests from the USB Host.
 * When a Setup request occurs, this task will launch the processing
 * of this setup contained in the usb_standard_request.c file.
 * Other class specific requests are also processed in this file.
 *
 * \par Application note:
 *      AVR1907: Xplain Evaluation Board
 *
 * \par Documentation
 *      For comprehensive code documentation, supported compilers, compiler
 *      settings and supported devices see readme.html
 *
 * \author
 *      Atmel Corporation: http://www.atmel.com \n
 *      Support email: avr@atmel.com
 *
 * $Revision: 3122 $
 * $Date: 2010-01-13 13:26:22 +0100 (on, 13 jan 2010) $  \n
 *
 * Copyright (c) 2010, Atmel Corporation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. The name of ATMEL may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ATMEL "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE EXPRESSLY AND
 * SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *****************************************************************************/

#include "config.h"
#include "conf_usb.h"
#include "usb_device_task.h"
#include "usb_task.h"
#include "usb_drv.h"
#include "usb_descriptors.h"
#include "usb_standard_request.h"

/*! \brief Global connection status byte
 *
 * usb_connected is set to true when VBUS has been detected
 * usb_connected is set to false otherwise
 */
uint8_t usb_connected;


/*! \brief Store the number of the USB configuration used by the USB device
 *
 *  When its value is different from zero, it means the device mode is enumerated
 */
extern uint8_t usb_configuration_nb;


/*! \brief This function initializes the USB device controller and system interrupt
 *
 *  This function enables the USB controller and init the USB interrupts.
 *  The aim is to allow the USB connection detection in order to send
 *  the appropriate USB event to the operating mode manager.
 */
void usb_device_task_init(void)
{
	sei();
	Usb_disable();
	Usb_enable();
	Usb_select_device();
	Usb_enable_vbus_interrupt();
	sei();
	
}

/*! \brief This function initializes the USB device controller
 *
 *  This function enables the USB controller and init the USB interrupts.
 *  The aim is to allow the USB connection detection in order to send
 *  the appropriate USB event to the operating mode manager.
 *  Start device function is executed once VBUS connection has been detected
 *  either by the VBUS change interrupt either by the VBUS high level
 */
void usb_start_device (void)
{
	Start_PLL();
	Wait_pll_ready();
	Usb_unfreeze_clock();
	Usb_enable_suspend_interrupt();
	Usb_enable_reset_interrupt();
	usb_init_device();               /* Configure the USB controller EP0 */
	Usb_attach();
	usb_connected = false;
}


/*! \brief Entry point of the USB device mamagement
 *
 *  This function is the entry point of the USB management. Each USB
 *  event is checked here in order to launch the appropriate action.
 *  If a Setup request occurs on the Default Control Endpoint,
 *  the usb_process_request() function is call in the usb_standard_request.c file
 */
void usb_device_task(void)
{
	if (Is_usb_vbus_high()&& usb_connected==false)
	{
		usb_connected = true;
		Usb_vbus_on_action();
		Usb_send_event(EVT_USB_POWERED);
		Usb_enable_reset_interrupt();
		Usb_attach();
	}
	
	if(Is_usb_event(EVT_USB_RESET))
	{
		Usb_ack_event(EVT_USB_RESET);
		Usb_reset_endpoint(0);
		usb_configuration_nb=0;
	}
	
	Usb_select_endpoint(EP_CONTROL);
	if (Is_usb_receive_setup())
	{
		usb_process_request();
	}
}
