/* This file has been prepared for Doxygen automatic documentation generation.*/
/*! \file *********************************************************************
 *
 * \brief This file contains the USB driver routines.
 *
 * \par Application note:
 *      AVR1907: Xplain Evaluation Board
 *
 * \par Documentation
 *      For comprehensive code documentation, supported compilers, compiler
 *      settings and supported devices see readme.html
 *
 * \author
 *      Atmel Corporation: http://www.atmel.com \n
 *      Support email: avr@atmel.com
 *
 * $Revision: 3122 $
 * $Date: 2010-01-13 13:26:22 +0100 (on, 13 jan 2010) $  \n
 *
 * Copyright (c) 2010, Atmel Corporation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. The name of ATMEL may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ATMEL "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE EXPRESSLY AND
 * SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *****************************************************************************/

#include "config.h"
#include "conf_usb.h"
#include "usb_drv.h"


/*! \brief This function configures an endpoint with the selected type.
 *
 *  \param config0
 *  \param config1
 *
 *  \retval Is_endpoint_configured.
 */
uint8_t usb_config_ep(uint8_t config0, uint8_t config1)
{
	Usb_enable_endpoint();
	UECFG0X = config0;
	UECFG1X = (UECFG1X & (1<<ALLOC)) | config1;
	Usb_allocate_memory();
	return (Is_endpoint_configured());
}


/*! \brief This function select the endpoint where an event occurs and returns the
 *         number of this endpoint. If no event occurs on the endpoints, this
 *         function returns 0.
 *
 *  \retval endpoint number.
 */
uint8_t usb_select_enpoint_interrupt(void)
{
	uint8_t interrupt_flags;
	uint8_t ep_num;
	
	ep_num = 0;
	interrupt_flags = Usb_interrupt_flags();
	
	while(ep_num < 9){
		if (interrupt_flags & 1){
			return (ep_num);
		}else{
			ep_num++;
			interrupt_flags = interrupt_flags >> 1;
		}
	}
	return 0;
}


/*! \brief This function moves the data pointed by tbuf to the selected endpoint fifo
 *         and sends it through the USB.
 *
 *  \note tbuf is incremented of 'data_length'.
 *
 *  \param ep_num       number of the addressed endpoint
 *  \param *tbuf        address of the first data to send
 *  \param data_length  number of bytes to send
 *
 *  \retval address of the next byte to send.
 */
uint8_t usb_send_packet(uint8_t ep_num, uint8_t* tbuf, uint8_t data_length)
{
	uint8_t remaining_length;
	
	remaining_length = data_length;
	Usb_select_endpoint(ep_num);
	while(Is_usb_write_enabled() && (0 != remaining_length)){
		Usb_write_byte(*tbuf);
		remaining_length--;
		tbuf++;
	}
	return remaining_length;
}

/*! \brief This function moves the data stored in the selected endpoint fifo to
 *         the address specified by *rbuf.
 *
 *  \note rbuf is incremented of 'data_length'.
 *
 *  \param ep_num       number of the addressed endpoint
 *  \param rbuf        aaddress of the first data to write with the USB data
 *  \param data_length  number of bytes to read
 *
 *  \retval address of the next uint8_t to send.
 */
uint8_t usb_read_packet(uint8_t ep_num, uint8_t* rbuf, uint8_t  data_length)
{
	uint8_t remaining_length;
	
	remaining_length = data_length;
	Usb_select_endpoint(ep_num);
	
	while(Is_usb_read_enabled() && (0 != remaining_length)){
		*rbuf = Usb_read_byte();
		remaining_length--;
		rbuf++;
	}
	return remaining_length;
}


/*! \brief This function sends a STALL handshake for the next Host request. A STALL
 *         handshake will be send for each next request untill a SETUP or a Clear Halt
 *         Feature occurs for this endpoint.
 *
 *  \param ep_num number of the addressed endpoint
 */
void usb_halt_endpoint (uint8_t ep_num)
{
	Usb_select_endpoint(ep_num);
	Usb_enable_stall_handshake();
}


/*! \brief This function initializes the USB device controller and
 *         configures the Default Control Endpoint.
 *
 *  \retval status
 */
uint8_t usb_init_device (void)
{
	Usb_select_device();
	if(Is_usb_id_device())
	{
		Usb_select_endpoint(EP_CONTROL);
		if(!Is_usb_endpoint_enabled())
		{
			return usb_configure_endpoint(EP_CONTROL,    \
			                              TYPE_CONTROL,  \
			                              DIRECTION_OUT, \
			                              SIZE_64,       \
			                              ONE_BANK,      \
			                              NYET_DISABLED);
		}
	}
	return false;
}

