/* This file has been prepared for Doxygen automatic documentation generation.*/
/*! \file *********************************************************************
 *
 * \brief This file contains the USB low level driver definition
 *
 * \par Application note:
 *      AVR1907: Xplain Evaluation Board
 *
 * \par Documentation
 *      For comprehensive code documentation, supported compilers, compiler
 *      settings and supported devices see readme.html
 *
 * \author
 *      Atmel Corporation: http://www.atmel.com \n
 *      Support email: avr@atmel.com
 *
 * $Revision: 3122 $
 * $Date: 2010-01-13 13:26:22 +0100 (on, 13 jan 2010) $  \n
 *
 * Copyright (c) 2010, Atmel Corporation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. The name of ATMEL may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ATMEL "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE EXPRESSLY AND
 * SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *****************************************************************************/


#ifndef _USB_DRV_H_
#define _USB_DRV_H_

/****************************** DEFINES ***************************************/

#define MAX_EP_NB             7

#define EP_CONTROL            0
#define EP_1                  1
#define EP_2                  2
#define EP_3                  3
#define EP_4                  4
#define EP_5                  5
#define EP_6                  6
#define EP_7                  7

#define PIPE_CONTROL          0
#define PIPE_0                0
#define PIPE_1                1
#define PIPE_2                2
#define PIPE_3                3
#define PIPE_4                4
#define PIPE_5                5
#define PIPE_6                6
#define PIPE_7                7

/* USB EndPoint Mask*/
#define MSK_EP_DIR            0x7F
#define MSK_UADD              0x7F
#define MSK_EPTYPE            0xC0
#define MSK_EPSIZE            0x70
#define MSK_EPBK              0x0C
#define MSK_DTSEQ             0x0C
#define MSK_NBUSYBK           0x03
#define MSK_CURRBK            0x03
#define MSK_DAT               0xFF  /* UEDATX */
#define MSK_BYCTH             0x07  /* UEBCHX */
#define MSK_BYCTL             0xFF  /* UEBCLX */
#define MSK_EPINT             0x7F  /* UEINT */
#define MSK_HADDR             0xFF  /* UHADDR */

/* USB Pipe Mask */
#define MSK_PNUM              0x07  /* UPNUM */
#define MSK_PRST              0x7F  /* UPRST */
#define MSK_PTYPE             0xC0  /* UPCFG0X */
#define MSK_PTOKEN            0x30
#define MSK_PEPNUM            0x0F

#define MSK_PSIZE             0x70  /* UPCFG1X */
#define MSK_PBK               0x0C
#define MSK_NBUSYBK           0x03
#define MSK_ERROR             0x1F

#define MSK_PTYPE             0xC0  /* UPCFG0X */
#define MSK_PTOKEN            0x30
#define MSK_TOKEN_SETUP       0x30
#define MSK_TOKEN_IN          0x10
#define MSK_TOKEN_OUT         0x20
#define MSK_PEPNUM            0x0F

#define MSK_PSIZE             0x70  /* UPCFG1X */
#define MSK_PBK               0x0C

/* Enums used to enable and configure an endpoint. */

typedef enum ep_type{
	TYPE_CONTROL        = 0x00,
	TYPE_ISOCHRONOUS    = 0x01,
	TYPE_BULK           = 0x02,
	TYPE_INTERRUPT      = 0x03
}e_ep_type;

typedef enum ep_dir{
	DIRECTION_OUT       = 0x00,
	DIRECTION_IN        = 0x01
}e_ep_dir;

typedef enum ep_size{
	SIZE_8              = 0x00,
	SIZE_16             = 0x01,
	SIZE_32             = 0x02,
	SIZE_64             = 0x03,
	SIZE_128            = 0x04,
	SIZE_256            = 0x05,
	SIZE_512            = 0x06,
	SIZE_1024           = 0x07
}e_ep_size;

typedef enum ep_bank{
	ONE_BANK            = 0x00,
	TWO_BANKS           = 0x01
}e_ep_bank;

typedef enum ep_nyet{
	NYET_DISABLED       = 0x00,
	NYET_ENABLED        = 0x01
}e_ep_nyet;

typedef enum ep_token{
	TOKEN_SETUP         = 0x00,
	TOKEN_IN            = 0x01,
	TOKEN_OUT           = 0x02
}e_ep_token;


/****************************** MACROS ****************************************/

#define Usb_build_ep_config0(type, dir, nyet)    ((type<<6) | (nyet<<1) | (dir))
#define Usb_build_ep_config1(size, bank)         ((size<<4) | (bank<<2))

#define usb_configure_endpoint(num, type, dir, size, bank, nyet)             \
              ( Usb_select_endpoint(num),                                    \
                usb_config_ep(Usb_build_ep_config0(type, dir, nyet),         \
                Usb_build_ep_config1(size, bank)    ))


/*! Enable internal USB pads regulator */
#define Usb_enable_regulator()          (UHWCON |= (1<<UVREGE))

/*! Disable internal USB pads regulator */
#define Usb_disable_regulator()         (UHWCON &= ~(1<<UVREGE))

/*! Check regulator enable bit */
#define Is_usb_regulator_enabled()      ((UHWCON &  (1<<UVREGE))  ? true : false)


/*! Enable external UID pin */
#define Usb_enable_uid_pin()            (UHWCON |= (1<<UIDE))

/*! Disable external UID pin */
#define Usb_disable_uid_pin()           (UHWCON &= ~(1<<UIDE))

/*! Disable external UID pin and force device mode */
#define Usb_force_device_mode()         (Usb_disable_uid_pin(), UHWCON |= (1<<UIMOD))

/*! Disable external UID pin and force host mode */
#define Usb_force_host_mode()           (Usb_disable_uid_pin(), UHWCON &= ~(1<<UIMOD))

/*! Enable external UVCON pin */
#define Usb_enable_uvcon_pin()          (UHWCON |= (1<<UVCONE))

/*! Enable external UVCON pin */
#define Usb_disable_uvcon_pin()         (UHWCON &= ~(1<<UVCONE))

/*! Use device full speed mode (default mode) */
#define Usb_full_speed_mode()           (UHWCON |= (1<<UDSS))

/*! For device mode, force low speed mode */
#define Usb_low_speed_mode()            (UHWCON &= ~(1<<UDSS))

/*! Enable both USB interface and Vbus pad */
#define Usb_enable()                  (USBCON |= ((1<<USBE) | (1<<OTGPADE)))

/*! Disable both USB interface and Vbus pad */
#define Usb_disable()                 (USBCON &= ~((1<<USBE) | (1<<OTGPADE)))
#define Is_usb_enabled()              ((USBCON  &   (1<<USBE))   ? true : false)

/*! Enable VBUS pad */
#define Usb_enable_vbus_pad()         (USBCON |= (1<<OTGPADE))

/*! Disable VBUS pad */
#define Usb_disable_vbus_pad()        (USBCON &= ~(1<<OTGPADE))

#define Usb_select_device()           (USBCON  &= ~(1<<HOST))
#define Usb_select_host()             (USBCON  |=  (1<<HOST))
#define Is_usb_host_enabled()         ((USBCON  &   (1<<HOST))   ? true : false)

/*! Stop internal USB clock in interface (freeze the interface register) */
#define Usb_freeze_clock()            (USBCON  |=  (1<<FRZCLK))
#define Usb_unfreeze_clock()          (USBCON  &= ~(1<<FRZCLK))
#define Is_usb_clock_freezed()        ((USBCON  &   (1<<FRZCLK)) ? true : false)

#define Usb_enable_id_interrupt()     (USBCON  |=  (1<<IDTE))
#define Usb_disable_id_interrupt()    (USBCON  &= ~(1<<IDTE))
#define Is_usb_id_interrupt_enabled() ((USBCON &  (1<<IDTE))     ? true : false)
#define Is_usb_id_device()            ((USBSTA &   (1<<ID))      ? true : false)
#define Usb_ack_id_transition()       (USBINT  = ~(1<<IDTI))
#define Is_usb_id_transition()        ((USBINT &   (1<<IDTI))    ? true : false)

#define Usb_enable_vbus_interrupt()   (USBCON  |=  (1<<VBUSTE))
#define Usb_disable_vbus_interrupt()  (USBCON  &= ~(1<<VBUSTE))
#define Is_usb_vbus_interrupt_enabled() ((USBCON &  (1<<VBUSTE))     ? true : false)
#define Is_usb_vbus_high()            ((USBSTA &   (1<<VBUS))    ? true : false)
#define Is_usb_vbus_low()             ((USBSTA &   (1<<VBUS))    ? false : true)
#define Usb_ack_vbus_transition()     (USBINT  = ~(1<<VBUSTI))
#define Is_usb_vbus_transition()      ((USBINT &   (1<<VBUSTI))  ? true : false)

/*! returns the USB general interrupts (interrupt enabled) */
#define Usb_get_general_interrupt()      (USBINT & (USBCON & MSK_IDTE_VBUSTE))

/*! acks the general interrupts (interrupt enabled) */
#define Usb_ack_all_general_interrupt()  (USBINT = ~(USBCON & MSK_IDTE_VBUSTE))
#define Usb_ack_cache_id_transition(x)   ((x)  &= ~(1<<IDTI))
#define Usb_ack_cache_vbus_transition(x) ((x)  &= ~(1<<VBUSTI))
#define Is_usb_cache_id_transition(x)    (((x) &   (1<<IDTI))  )
#define Is_usb_cache_vbus_transition(x)  (((x) &   (1<<VBUSTI)))

/*! returns the USB Pad interrupts (interrupt enabled) */
#define Usb_get_otg_interrupt()            (OTGINT & OTGIEN)

/*! acks the USB Pad interrupts (interrupt enabled) */
#define Usb_ack_all_otg_interrupt()        (OTGINT = ~OTGIEN)
#define Is_otg_cache_bconnection_error(x)  (((x) &   MSK_BCERRI))
#define Usb_ack_cache_bconnection_error(x) ((x)  &= ~MSK_BCERRI)

#define Usb_enter_dpram_mode()        (UDPADDH =  (1<<DPACC))
#define Usb_exit_dpram_mode()         (UDPADDH =  (uint8_t)~(1<<DPACC))
#define Usb_set_dpram_address(addr)   (UDPADDH =  (1<<DPACC) + ((Uint16)addr >> 8), UDPADDL = (uint8_t)addr)
#define Usb_write_dpram_byte(val)     (UEDATX=val)
#define Usb_read_dpram_byte()			  (UEDATX)

/*! requests for VBus activation */
#define Usb_enable_vbus()             (OTGCON  |=  (1<<VBUSREQ))

/*! requests for VBus desactivation */
#define Usb_disable_vbus()            (OTGCON  |=  (1<<VBUSRQC))

/*! Manually request VBUS without UVCON signal from USB interface */
#define Usb_enable_manual_vbus()      (PORTE|=0x80,DDRE|=0x80,Usb_disable_uvcon_pin())

/*! initiates a Host Negociation Protocol */
#define Usb_device_initiate_hnp()     (OTGCON  |=  (1<<HNPREQ))

/*! accepts a Host Negociation Protocol */
#define Usb_host_accept_hnp()         (OTGCON  |=  (1<<HNPREQ))

/*! rejects a Host Negociation Protocol */
#define Usb_host_reject_hnp()         (OTGCON  &= ~(1<<HNPREQ))

/*! initiates a Session Request Protocol */
#define Usb_device_initiate_srp()     (OTGCON  |=  (1<<SRPREQ))

/*! selects VBus as SRP method */
#define Usb_select_vbus_srp_method()  (OTGCON  |=  (1<<SRPSEL))

/*! selects data line as SRP method */
#define Usb_select_data_srp_method()  (OTGCON  &= ~(1<<SRPSEL))

/*! enables hardware control on VBus */
#define Usb_enable_vbus_hw_control()  (OTGCON  &= ~(1<<VBUSHWC))

/*! disables hardware control on VBus */
#define Usb_disable_vbus_hw_control() (OTGCON  |=  (1<<VBUSHWC))

/*! tests if VBus has been requested */
#define Is_usb_vbus_enabled()         ((OTGCON &   (1<<VBUSREQ)) ? true : false)

/*! tests if a HNP occurs */
#define Is_usb_hnp()                  ((OTGCON &   (1<<HNPREQ))  ? true : false)

/*! tests if a SRP from device occurs */
#define Is_usb_device_srp()           ((OTGCON &   (1<<SRPREQ))  ? true : false)

/*! enables suspend time out interrupt */
#define Usb_enable_suspend_time_out_interrupt()   (OTGIEN  |=  (1<<STOE))

/*! disables suspend time out interrupt */
#define Usb_disable_suspend_time_out_interrupt()  (OTGIEN  &= ~(1<<STOE))
#define Is_suspend_time_out_interrupt_enabled()   ((OTGIEN &  (1<<STOE))   ? true : false)

/*! acks suspend time out interrupt */
#define Usb_ack_suspend_time_out_interrupt()      (OTGINT  &= ~(1<<STOI))

/*! tests if a suspend time out occurs */
#define Is_usb_suspend_time_out_interrupt()       ((OTGINT &   (1<<STOI))    ? true : false)

/*! enables HNP error interrupt */
#define Usb_enable_hnp_error_interrupt()          (OTGIEN  |=  (1<<HNPERRE))

/*! disables HNP error interrupt */
#define Usb_disable_hnp_error_interrupt()         (OTGIEN  &= ~(1<<HNPERRE))

#define Is_hnp_error_interrupt_enabled()          ((OTGIEN &  (1<<HNPERRE))   ? true : false)

/*! acks HNP error interrupt */
#define Usb_ack_hnp_error_interrupt()             (OTGINT  &= ~(1<<HNPERRI))

/*! tests if a HNP error occurs */
#define Is_usb_hnp_error_interrupt()              ((OTGINT &   (1<<HNPERRI)) ? true : false)

/*! enables role exchange interrupt */
#define Usb_enable_role_exchange_interrupt()      (OTGIEN  |=  (1<<ROLEEXE))

/*! disables role exchange interrupt */
#define Usb_disable_role_exchange_interrupt()     (OTGIEN  &= ~(1<<ROLEEXE))
#define Is_role_exchange_interrupt_enabled()      ((OTGIEN &  (1<<ROLEEXE))   ? true : false)

/*! acks role exchange interrupt */
#define Usb_ack_role_exchange_interrupt()         (OTGINT  &= ~(1<<ROLEEXI))

/*! tests if a role exchange occurs */
#define Is_usb_role_exchange_interrupt()          ((OTGINT &   (1<<ROLEEXI)) ? true : false)

/*! enables B device connection error interrupt */
#define Usb_enable_bconnection_error_interrupt()  (OTGIEN  |=  (1<<BCERRE))

/*! disables B device connection error interrupt */
#define Usb_disable_bconnection_error_interrupt() (OTGIEN  &= ~(1<<BCERRE))
#define Is_bconnection_error_interrupt_enabled()  ((OTGIEN &  (1<<BCERRE))   ? true : false)

/*! acks B device connection error interrupt */
#define Usb_ack_bconnection_error_interrupt()     (OTGINT  &= ~(1<<BCERRI))

/*! tests if a B device connection error occurs */
#define Is_usb_bconnection_error_interrupt()      ((OTGINT &   (1<<BCERRI))  ? true : false)

/*! enables VBus error interrupt */
#define Usb_enable_vbus_error_interrupt()         (OTGIEN  |=  (1<<VBERRE))

/*! disables VBus error interrupt */
#define Usb_disable_vbus_error_interrupt()        (OTGIEN  &= ~(1<<VBERRE))
#define Is_vbus_error_interrupt_enabled()         ((OTGIEN &  (1<<VBERRE))   ? true : false)

/*! acks VBus error interrupt */
#define Usb_ack_vbus_error_interrupt()            (OTGINT  &= ~(1<<VBERRI))

/*! tests if a VBus error occurs */
#define Is_usb_vbus_error_interrupt()             ((OTGINT &   (1<<VBERRI))  ? true : false)

/*! enables SRP interrupt */
#define Usb_enable_srp_interrupt()                (OTGIEN  |=  (1<<SRPE))

/*! disables SRP interrupt */
#define Usb_disable_srp_interrupt()               (OTGIEN  &= ~(1<<SRPE))
#define Is_srp_interrupt_enabled()                ((OTGIEN &  (1<<SRPE))   ? true : false)

/*! acks SRP interrupt */
#define Usb_ack_srp_interrupt()                   (OTGINT  &= ~(1<<SRPI))

/*! tests if a SRP occurs */
#define Is_usb_srp_interrupt()                    ((OTGINT &   (1<<SRPI))    ? true : false)



/*! initiates a remote wake-up */
#define Usb_initiate_remote_wake_up()             (UDCON   |=  (1<<RMWKUP))

/*! detaches from USB bus */
#define Usb_detach()                              (UDCON   |=  (1<<DETACH))

/*! attaches to USB bus */
#define Usb_attach()                              (UDCON   &= ~(1<<DETACH))

/*! test if remote wake-up still running */
#define Is_usb_pending_remote_wake_up()           ((UDCON & (1<<RMWKUP)) ? true : false)

/*! test if the device is detached */
#define Is_usb_detached()                         ((UDCON & (1<<DETACH)) ? true : false)

/*! returns the USB device interrupts (interrupt enabled) */
#define Usb_get_device_interrupt()                (UDINT   &   (1<<UDIEN))

/*! acks the USB device interrupts (interrupt enabled) */
#define Usb_ack_all_device_interrupt()            (UDINT   =  ~(1<<UDIEN))

/*! enables remote wake-up interrupt */
#define Usb_enable_remote_wake_up_interrupt()     (UDIEN   |=  (1<<UPRSME))

/*! disables remote wake-up interrupt */
#define Usb_disable_remote_wake_up_interrupt()    (UDIEN   &= ~(1<<UPRSME))
#define Is_remote_wake_up_interrupt_enabled()     ((UDIEN &  (1<<UPRSME))   ? true : false)

/*! acks remote wake-up */
#define Usb_ack_remote_wake_up_start()            (UDINT   = ~(1<<UPRSMI))

/*! tests if remote wake-up still running */
#define Is_usb_remote_wake_up_start()             ((UDINT &   (1<<UPRSMI))  ? true : false)

/*! enables resume interrupt */
#define Usb_enable_resume_interrupt()             (UDIEN   |=  (1<<EORSME))

/*! disables resume interrupt */
#define Usb_disable_resume_interrupt()            (UDIEN   &= ~(1<<EORSME))
#define Is_resume_interrupt_enabled()             ((UDIEN &  (1<<EORSME))   ? true : false)

/*! acks resume */
#define Usb_ack_resume()                          (UDINT   = ~(1<<EORSMI))

/*! tests if resume occurs */
#define Is_usb_resume()                           ((UDINT &   (1<<EORSMI))  ? true : false)

/*! enables wake-up interrupt */
#define Usb_enable_wake_up_interrupt()            (UDIEN   |=  (1<<WAKEUPE))

/*! disables wake-up interrupt */
#define Usb_disable_wake_up_interrupt()           (UDIEN   &= ~(1<<WAKEUPE))
#define Is_swake_up_interrupt_enabled()           ((UDIEN &  (1<<WAKEUPE))   ? true : false)

/*! acks wake-up */
#define Usb_ack_wake_up()                         (UDINT   = ~(1<<WAKEUPI))

/*! tests if wake-up occurs */
#define Is_usb_wake_up()                          ((UDINT &   (1<<WAKEUPI)) ? true : false)

/*! enables USB reset interrupt */
#define Usb_enable_reset_interrupt()              (UDIEN   |=  (1<<EORSTE))

/*! disables USB reset interrupt */
#define Usb_disable_reset_interrupt()             (UDIEN   &= ~(1<<EORSTE))
#define Is_reset_interrupt_enabled()              ((UDIEN &  (1<<EORSTE))   ? true : false)

/*! acks USB reset */
#define Usb_ack_reset()                           (UDINT   = ~(1<<EORSTI))

/*! tests if USB reset occurs */
#define Is_usb_reset()                            ((UDINT &   (1<<EORSTI))  ? true : false)

/*! enables Start Of Frame Interrupt */
#define Usb_enable_sof_interrupt()                (UDIEN   |=  (1<<SOFE))

/*! disables Start Of Frame Interrupt */
#define Usb_disable_sof_interrupt()               (UDIEN   &= ~(1<<SOFE))
#define Is_sof_interrupt_enabled()                ((UDIEN &  (1<<SOFE))   ? true : false)

/*! acks Start Of Frame */
#define Usb_ack_sof()                             (UDINT   = ~(1<<SOFI))

/*! tests if Start Of Frame occurs */
#define Is_usb_sof()                              ((UDINT &   (1<<SOFI))    ? true : false)


/*! enables suspend state interrupt */
#define Usb_enable_suspend_interrupt()            (UDIEN   |=  (1<<SUSPE))

/*! disables suspend state interrupt */
#define Usb_disable_suspend_interrupt()           (UDIEN   &= ~(1<<SUSPE))
#define Is_suspend_interrupt_enabled()            ((UDIEN &  (1<<SUSPE))   ? true : false)

/*! acks Suspend */
#define Usb_ack_suspend()                         (UDINT   = ~(1<<SUSPI))

/*! tests if Suspend state detected */
#define Is_usb_suspend()                          ((UDINT &   (1<<SUSPI))   ? true : false)

/*! enables USB device address */
#define Usb_enable_address()                      (UDADDR  |=  (1<<ADDEN))

/*! disables USB device address */
#define Usb_disable_address()                     (UDADDR  &= ~(1<<ADDEN))

/*! sets the USB device address */
#define Usb_configure_address(addr)               (UDADDR  =   (UDADDR & (1<<ADDEN)) | ((uint8_t)addr & MSK_UADD))

/*! returns the last frame number */
#define Usb_frame_number()                        ((uint16_t)((((uint16_t)UDFNUMH) << 8) | ((uint16_t)UDFNUML)))

/*! tests if a crc error occurs in frame number */
#define Is_usb_frame_number_crc_error()           ((UDMFN & (1<<FNCERR)) ? true : false)

/*! selects the endpoint number to interface with the CPU */
#define Usb_select_endpoint(ep)                   (UENUM = (uint8_t)ep )

/*! resets the selected endpoint */
#define Usb_reset_endpoint(ep)                    (UERST   =   1 << (uint8_t)ep, UERST  =  0)

/*! enables the current endpoint */
#define Usb_enable_endpoint()                     (UECONX  |=  (1<<EPEN))

/*! enables the STALL handshake for the next transaction */
#define Usb_enable_stall_handshake()              (UECONX  |=  (1<<STALLRQ))

/*! resets the data toggle sequence */
#define Usb_reset_data_toggle()                   (UECONX  |=  (1<<RSTDT))

/*! disables the current endpoint */
#define Usb_disable_endpoint()                    (UECONX  &= ~(1<<EPEN))

/*! disables the STALL handshake */
#define Usb_disable_stall_handshake()             (UECONX  |=  (1<<STALLRQC))

/*! selects endpoint interface on CPU */
#define Usb_select_epnum_for_cpu()                (UECONX  &= ~(1<<EPNUMS))
   /*! tests if the current endpoint is enabled */
#define Is_usb_endpoint_enabled()                 ((UECONX & (1<<EPEN))    ? true : false)

/*! tests if STALL handshake request is running */
#define Is_usb_endpoint_stall_requested()         ((UECONX & (1<<STALLRQ)) ? true : false)


/*! configures the current endpoint */
#define Usb_configure_endpoint_type(type)         (UECFG0X =   (UECFG0X & ~(MSK_EPTYPE)) | ((uint8_t)type << 6))

/*! configures the current endpoint direction */
#define Usb_configure_endpoint_direction(dir)     (UECFG0X =   (UECFG0X & ~(1<<EPDIR))  | ((uint8_t)dir))


/*! configures the current endpoint size */
#define Usb_configure_endpoint_size(size)         (UECFG1X =   (UECFG1X & ~MSK_EPSIZE) | ((uint8_t)size << 4))

/*! configures the current endpoint number of banks */
#define Usb_configure_endpoint_bank(bank)         (UECFG1X =   (UECFG1X & ~MSK_EPBK)   | ((uint8_t)bank << 2))

/*! allocates the current configuration in DPRAM memory */
#define Usb_allocate_memory()                     (UECFG1X |=  (1<<ALLOC))

/*! un-allocates the current configuration in DPRAM memory */
#define Usb_unallocate_memory()                   (UECFG1X &= ~(1<<ALLOC))

/*! acks endpoint overflow interrupt */
#define Usb_ack_overflow_interrupt()              (UESTA0X &= ~(1<<OVERFI))

/*! acks endpoint underflow memory */
#define Usb_ack_underflow_interrupt()             (UESTA0X &= ~(1<<UNDERFI))

/*! acks Zero Length Packet received */
#define Usb_ack_zlp()                             (UESTA0X &= ~(1<<ZLPSEEN))

/*! returns data toggle */
#define Usb_data_toggle()                         ((UESTA0X&MSK_DTSEQ) >> 2)

/*! returns the number of busy banks */
#define Usb_nb_busy_bank()                        (UESTA0X &   MSK_NBUSYBK)

/*! tests if at least one bank is busy */
#define Is_usb_one_bank_busy()                    ((UESTA0X &  MSK_NBUSYBK) == 0 ? false : true)

/*! tests if current endpoint is configured */
#define Is_endpoint_configured()                  ((UESTA0X &  (1<<CFGOK))   ? true : false)

/*! tests if an overflows occurs */
#define Is_usb_overflow()                         ((UESTA0X &  (1<<OVERFI))  ? true : false)

/*! tests if an underflow occurs */
#define Is_usb_underflow()                        ((UESTA0X &  (1<<UNDERFI)) ? true : false)

/*! tests if a ZLP has been detected */
#define Is_usb_zlp()                              ((UESTA0X &  (1<<ZLPSEEN)) ? true : false)

   /*! returns the control direction */
#define Usb_control_direction()                   ((UESTA1X &  (1<<CTRLDIR)) >> 2)
   /*! returns the number of the current bank */
#define Usb_current_bank()                        ( UESTA1X & MSK_CURRBK)

/*! clears FIFOCON bit */
#define Usb_ack_fifocon()                         (UEINTX &= ~(1<<FIFOCON))

/*! acks NAK IN received */
#define Usb_ack_nak_in()                          (UEINTX &= ~(1<<NAKINI))

/*! acks NAK OUT received */
#define Usb_ack_nak_out()                         (UEINTX &= ~(1<<NAKOUTI))

/*! acks receive SETUP */
#define Usb_ack_receive_setup()                   (UEINTX &= ~(1<<RXSTPI))

/*! acks reveive OUT */
#define Usb_ack_receive_out()                     (UEINTX &= ~(1<<RXOUTI), Usb_ack_fifocon())

/*! acks STALL sent */
#define Usb_ack_stalled()                         (MSK_STALLEDI=   0)

/*! acks IN ready */
#define Usb_ack_in_ready()                        (UEINTX &= ~(1<<TXINI), Usb_ack_fifocon())

/*! Kills last bank */
#define Usb_kill_last_in_bank()                   (UENTTX |= (1<<RXOUTI))

/*! tests if endpoint read allowed */
#define Is_usb_read_enabled()                     (UEINTX&(1<<RWAL))

/*! tests if endpoint write allowed */
#define Is_usb_write_enabled()                    (UEINTX&(1<<RWAL))

/*! tests if read allowed on control endpoint */
#define Is_usb_read_control_enabled()             (UEINTX&(1<<TXINI))

/*! tests if SETUP received */
#define Is_usb_receive_setup()                    (UEINTX&(1<<RXSTPI))

/*! tests if OUT received */
#define Is_usb_receive_out()                      (UEINTX&(1<<RXOUTI))

/*! tests if IN ready */
#define Is_usb_in_ready()                         (UEINTX&(1<<TXINI))

/*! sends IN */
#define Usb_send_in()                             (UEINTX &= ~(1<<FIFOCON))

/*! sends IN on control endpoint */
#define Usb_send_control_in()                     (UEINTX &= ~(1<<TXINI))

/*! frees OUT bank */
#define Usb_free_out_bank()                       (UEINTX &= ~(1<<FIFOCON))

/*! acks OUT on control endpoint */
#define Usb_ack_control_out()                     (UEINTX &= ~(1<<RXOUTI))

/*! enables flow error interrupt */
#define Usb_enable_flow_error_interrupt()         (UEIENX  |=  (1<<FLERRE))

/*! enables NAK IN interrupt */
#define Usb_enable_nak_in_interrupt()             (UEIENX  |=  (1<<NAKINE))

/*! enables NAK OUT interrupt */
#define Usb_enable_nak_out_interrupt()            (UEIENX  |=  (1<<NAKOUTE))

/*! enables receive SETUP interrupt */
#define Usb_enable_receive_setup_interrupt()      (UEIENX  |=  (1<<RXSTPE))

/*! enables receive OUT interrupt */
#define Usb_enable_receive_out_interrupt()        (UEIENX  |=  (1<<RXOUTE))

/*! enables STALL sent interrupt */
#define Usb_enable_stalled_interrupt()            (UEIENX  |=  (1<<STALLEDE))

/*! enables IN ready interrupt */
#define Usb_enable_in_ready_interrupt()           (UEIENX  |=  (1<<TXIN))

/*! disables flow error interrupt */
#define Usb_disable_flow_error_interrupt()        (UEIENX  &= ~(1<<FLERRE))

/*! disables NAK IN interrupt */
#define Usb_disable_nak_in_interrupt()            (UEIENX  &= ~(1<<NAKINE))

/*! disables NAK OUT interrupt */
#define Usb_disable_nak_out_interrupt()           (UEIENX  &= ~(1<<NAKOUTE))

/*! disables receive SETUP interrupt */
#define Usb_disable_receive_setup_interrupt()     (UEIENX  &= ~(1<<RXSTPE))

/*! disables receive OUT interrupt */
#define Usb_disable_receive_out_interrupt()       (UEIENX  &= ~(1<<RXOUTE))

/*! disables STALL sent interrupt */
#define Usb_disable_stalled_interrupt()           (UEIENX  &= ~(1<<STALLEDE))

/*! disables IN ready interrupt */
#define Usb_disable_in_ready_interrupt()          (UEIENX  &= ~(1<<TXIN))

/*! returns FIFO byte for current endpoint */
#define Usb_read_byte()                           (UEDATX)

/*! writes byte in FIFO for current endpoint */
#define Usb_write_byte(byte)                      (UEDATX  =   (uint8_t)byte)

/*! returns number of bytes in FIFO current endpoint (16 bits) */
#define Usb_byte_counter()                        ((((uint16_t)UEBCHX) << 8) | (UEBCLX))

/*! returns number of bytes in FIFO current endpoint (8 bits) */
#define Usb_byte_counter_8()                      ((uint8_t)UEBCLX)

/*! tests the general endpoint interrupt flags */
#define Usb_interrupt_flags()                     (UEINT != 0x00)

/*! tests the general endpoint interrupt flags */
#define Is_usb_endpoint_event()                   (Usb_interrupt_flags())

/* ADVANCED MACROS */
#define Usb_select_ep_for_cpu(ep)                 (Usb_select_epnum_for_cpu(), Usb_select_endpoint(ep))
#define Get_pipe_token(x)                         ((x & (0x80)) ? TOKEN_IN : TOKEN_OUT)

/*! This macro help to fill the uint16_t fill in USB enumeration struct. */
#define Usb_write_word_enum_struc(x)	(x)


/************************* PROTOTYPES (GLOBAL) ********************************/

uint8_t  usb_config_ep(uint8_t, uint8_t);
uint8_t  usb_select_enpoint_interrupt(void);
uint16_t usb_get_nb_byte_epw(void);
uint8_t  usb_send_packet(uint8_t , uint8_t*, uint8_t);
uint8_t  usb_read_packet(uint8_t , uint8_t*, uint8_t);
void     usb_halt_endpoint(uint8_t);
void     usb_reset_endpoint(uint8_t);
uint8_t  usb_init_device(void);

#endif  /* _USB_DRV_H_ */

