/* This file has been prepared for Doxygen automatic documentation generation.*/
/*! \file *********************************************************************
 *
 * \brief User call-back functions
 *
 * This file contains the user call-back functions corresponding to the
 * application: CDC
 *
 * \par Application note:
 *      AVR1907: Xplain Evaluation Board
 *
 * \par Documentation
 *      For comprehensive code documentation, supported compilers, compiler
 *      settings and supported devices see readme.html
 *
 * \author
 *      Atmel Corporation: http://www.atmel.com \n
 *      Support email: avr@atmel.com
 *
 * $Revision: 3119 $
 * $Date: 2010-01-12 15:53:30 +0100 (ti, 12 jan 2010) $  \n
 *
 * Copyright (c) 2010, Atmel Corporation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. The name of ATMEL may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ATMEL "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE EXPRESSLY AND
 * SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *****************************************************************************/

#include "config.h"
#include "conf_usb.h"
#include "usb_drv.h"
#include "usb_descriptors.h"
#include "usb_standard_request.h"
#include "usb_specific_request.h"
#include "sw_usart.h"

/* External dependencies. */
extern uint8_t code *pbuffer;
extern uint8_t data_to_transfer;
extern S_line_coding line_coding;


/*! \brief This function is called by the standard usb read request function when
 *         the Usb request is not supported.
 *
 *  This function returns true when the request is processed.
 *  This function returns false if the request is not supported.
 *  In this case, a STALL handshake will be automatically sent by the standard
 *  usb read request function.
 */
bool usb_user_read_request(uint8_t type, uint8_t request)
{
	bool     status = true;
	uint8_t  descriptor_type ;
	uint8_t  string_type     ;
	
	string_type     = Usb_read_byte();
	descriptor_type = Usb_read_byte();
	switch(request)
	{
	    case GET_LINE_CODING:
		cdc_get_line_coding();
		break;
	    case SET_LINE_CODING:
		cdc_set_line_coding();
		break;
	    case SET_CONTROL_LINE_STATE:
		cdc_set_control_line_state();
		break;
	    default:
		status = false;
		break;
	}
	return status;
}


/*! \brief This function returns the size and the pointer on a user information
 *         structure
 *
 *  \param type of the request
 *  \param string with the request desciption
 *
 *  \return bool True if type and string is valid, false otherwise.
 */
bool usb_user_get_descriptor(uint8_t type, uint8_t string)
{
	bool status = true;
	switch(type)
	{
	    case STRING_DESCRIPTOR:
		switch (string)
		{
		    case LANG_INDEX:
          		data_to_transfer = sizeof (usb_user_language_id);
          		pbuffer = &(usb_user_language_id.bLength);
          		break;
		    case MAN_INDEX:
         	 	data_to_transfer = sizeof (usb_user_manufacturer_string_descriptor);
         	 	pbuffer = &(usb_user_manufacturer_string_descriptor.bLength);
          		break;
		    case PROD_INDEX:
         		data_to_transfer = sizeof (usb_user_product_string_descriptor);
          		pbuffer = &(usb_user_product_string_descriptor.bLength);
          		break;
			
/* Include this if serial number should be fixed and not given by OS. */
/*		    case SN_INDEX:
          		data_to_transfer = sizeof (usb_user_serial_number);
          		pbuffer = &(usb_user_serial_number.bLength);
          		break;
*/		    case IF0_INDEX:
          		data_to_transfer = sizeof (usb_if0_descriptor);
          		pbuffer = &(usb_if0_descriptor.bLength);
          		break;
		    default:
          		status = false;
			break;
		}
		break;
	    default:
		status = false;
		break;
	}
	
	return status;
}

/*! \brief This function configures the endpoints.
 *
 *  \param conf_nb Configuration number
 */
void usb_user_endpoint_init(uint8_t conf_nb)
{
	usb_configure_endpoint(INT_EP,           \
	                       TYPE_INTERRUPT,   \
	                       DIRECTION_IN,     \
	                       SIZE_32,          \
	                       ONE_BANK,         \
	                       NYET_ENABLED);
	
	usb_configure_endpoint(TX_EP,            \
	                       TYPE_BULK,        \
	                       DIRECTION_IN,     \
	                       SIZE_32,          \
	                       ONE_BANK,         \
	                       NYET_ENABLED);
	
	usb_configure_endpoint(RX_EP,            \
	                       TYPE_BULK,        \
	                       DIRECTION_OUT,    \
	                       SIZE_32,          \
	                       TWO_BANKS,        \
	                       NYET_ENABLED);
	
	Usb_reset_endpoint(INT_EP);
	Usb_reset_endpoint(TX_EP);
	Usb_reset_endpoint(RX_EP);
}

/*! \brief This function manages reception of line coding parameters (baudrate...).
 */
void cdc_get_line_coding(void)
{
  	Usb_ack_receive_setup();
  	Usb_write_byte(LSB0(line_coding.dwDTERate));
  	Usb_write_byte(LSB1(line_coding.dwDTERate));
  	Usb_write_byte(LSB2(line_coding.dwDTERate));
  	Usb_write_byte(LSB3(line_coding.dwDTERate));
  	Usb_write_byte(line_coding.bCharFormat);
  	Usb_write_byte(line_coding.bParityType);
  	Usb_write_byte(line_coding.bDataBits);
	
  	Usb_send_control_in();
  	while(!(Is_usb_read_control_enabled()));
	
	while(!Is_usb_receive_out());
	Usb_ack_receive_out();
}


/*! \brief This function manages reception of line coding parameters (baudrate...).
 */
void cdc_set_line_coding (void)
{
	Usb_ack_receive_setup();
	while (!(Is_usb_receive_out()));
	
	LSB0(line_coding.dwDTERate) = Usb_read_byte();
	LSB1(line_coding.dwDTERate) = Usb_read_byte();
	LSB2(line_coding.dwDTERate) = Usb_read_byte();
	LSB3(line_coding.dwDTERate) = Usb_read_byte();
	
	line_coding.bCharFormat = Usb_read_byte();
	line_coding.bParityType = Usb_read_byte();
	line_coding.bDataBits = Usb_read_byte();
  	Usb_ack_receive_out();
	
	/* Send a ZLP for STATUS phase */
  	Usb_send_control_in();
  	while(!(Is_usb_read_control_enabled()));
	SW_USART_Set_Baudrate(line_coding.dwDTERate);
}


/*! \brief This function manages the SET_CONTROL_LINE_LINE_STATE CDC request.
 */
void cdc_set_control_line_state (void)
{
  	Usb_ack_receive_setup();
	Usb_send_control_in();
  	while(!(Is_usb_read_control_enabled()));

}
