/* This file has been prepared for Doxygen automatic documentation generation.*/
/*! \file *********************************************************************
 *
 * \brief Process USB device enumeration requests.
 *
 * This file contains the USB endpoint 0 management routines corresponding to
 * the standard enumeration process (refer to chapter 9 of the USB
 * specification.
 * This file calls routines of the usb_specific_request.c file for non-standard
 * request management.
 * The enumeration parameters (descriptor tables) are contained in the
 * usb_descriptors.c file.
 *
 * \par Application note:
 *      AVR1907: Xplain Evaluation Board
 *
 * \par Documentation
 *      For comprehensive code documentation, supported compilers, compiler
 *      settings and supported devices see readme.html
 *
 * \author
 *      Atmel Corporation: http://www.atmel.com \n
 *      Support email: avr@atmel.com
 *
 * $Revision: 3122 $
 * $Date: 2010-01-13 13:26:22 +0100 (on, 13 jan 2010) $  \n
 *
 * Copyright (c) 2010, Atmel Corporation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. The name of ATMEL may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ATMEL "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE EXPRESSLY AND
 * SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *****************************************************************************/


#include "config.h"
#include "conf_usb.h"
#include "usb_drv.h"
#include "usb_descriptors.h"
#include "usb_standard_request.h"
#include "usb_specific_request.h"

/************************* PROTOTYPES (LOCAL) ********************************/

static void usb_get_descriptor(void);
static void usb_set_address(void);
static void usb_set_configuration(void);
static void usb_clear_feature(void);
static void usb_set_feature(void);
static void usb_get_status(void);
static void usb_get_configuration(void);
static void usb_get_interface (void);
static void usb_set_interface (void);


/* Local parameters. */

static  uint8_t zlp;
static  uint8_t endpoint_status[NB_ENDPOINTS];

uint8_t code * pbuffer;
uint8_t data_to_transfer;
uint16_t wInterface;
static uint8_t bmRequestType;
uint8_t usb_configuration_nb;
extern uint8_t usb_connected;
extern code S_usb_device_descriptor usb_user_device_descriptor;
extern code S_usb_user_configuration_descriptor usb_user_configuration_descriptor;




/*! \brief This function initializes the main usb variables:
 *  - endpoint status
 *  - connection status
 */
void usb_enum_var_init (void)
{
	uint8_t ep_num ;
	
	for( ep_num=0 ; ep_num<NB_ENDPOINTS ; ep_num++ ){
		endpoint_status[ep_num] = 0;
	}
	usb_connected        = false;    /* USB is not connected */
	usb_configuration_nb = 0;        /* Default configuration number is 0 */
}


/*! \brief This function reads the SETUP request sent to the default control endpoint
 *  and calls the appropriate function. When exiting of the usb_read_request
 *  function, the device is ready to manage the next request.
 *
 *  \note list of supported requests:
 *  GET_DESCRIPTOR
 *  GET_CONFIGURATION
 *  SET_ADDRESS
 *  SET_CONFIGURATION
 *  CLEAR_FEATURE
 *  SET_FEATURE
 *  GET_STATUS
 */
void usb_process_request(void)
{
	uint8_t  bmRequest;
	
	bmRequestType = Usb_read_byte();
	bmRequest     = Usb_read_byte();
	
	switch (bmRequest)
	{
	    case GET_DESCRIPTOR:
		if (0x80 == bmRequestType) { usb_get_descriptor(); }
		else                       { usb_user_read_request(bmRequestType, bmRequest); }
		break;
		
	    case GET_CONFIGURATION:
		if (0x80 == bmRequestType) { usb_get_configuration(); }
		else                       { usb_user_read_request(bmRequestType, bmRequest); }
		break;
		
	    case SET_ADDRESS:
		if (0x00 == bmRequestType) { usb_set_address(); }
		else                       { usb_user_read_request(bmRequestType, bmRequest); }
		break;
		
	    case SET_CONFIGURATION:
		if (0x00 == bmRequestType) { usb_set_configuration(); }
		else                       { usb_user_read_request(bmRequestType, bmRequest); }
		break;
		
	    case CLEAR_FEATURE:
		if (0x02 >= bmRequestType) { usb_clear_feature(); }
		else                       { usb_user_read_request(bmRequestType, bmRequest); }
		break;
		
	    case SET_FEATURE:
		if (0x02 >= bmRequestType) { usb_set_feature(); }
		else                       { usb_user_read_request(bmRequestType, bmRequest); }
		break;
		
	    case GET_STATUS:
		if ((0x7F < bmRequestType) & (0x82 >= bmRequestType))
		{ usb_get_status(); }
		else                       { usb_user_read_request(bmRequestType, bmRequest); }
		break;
		
	    case GET_INTERFACE:
		if (bmRequestType == 0x81) { usb_get_interface(); }
		else { usb_user_read_request(bmRequestType, bmRequest); }
		break;
		
		
	    case SET_INTERFACE:
		if (bmRequestType == 0x01) {usb_set_interface();}
		break;
	
	    /* Un-supported request => call to user read request */
	    case SET_DESCRIPTOR:
	    case SYNCH_FRAME:
	    default:
		if(usb_user_read_request(bmRequestType, bmRequest) == false)
		{
			Usb_enable_stall_handshake();
			Usb_ack_receive_setup();
		}
		break;
	}
}



/*! \brief This function manages the SET ADDRESS request. When complete, the device
 *         will filter the requests using the new address.
 */
void usb_set_address(void)
{
#ifdef __ICCAVR__
#pragma diag_suppress=Pa082 /* Suppress warning for undefined order of volatile access. */
#endif
	Usb_configure_address(Usb_read_byte());
#ifdef __ICCAVR__
#pragma diag_default=Pa082 /* Back to default. */
#endif


	
	Usb_ack_receive_setup();
	
	/* Send a ZLP for STATUS phase */
	Usb_send_control_in();
	while(!Is_usb_in_ready());

	Usb_enable_address();
}


/*! \brief This function manages the SET CONFIGURATION request. If the selected
 *  configuration is valid, this function call the usb_user_endpoint_init()
 *  function that will configure the endpoints following the configuration
 *  number.
 */
void usb_set_configuration( void )
{
	uint8_t configuration_number;
	
	configuration_number = Usb_read_byte();
	
	if (configuration_number <= NB_CONFIGURATION){
		Usb_ack_receive_setup();
		usb_configuration_nb = configuration_number;
	}else{
		/* Keep that order (set StallRq/clear RxSetup) or a OUT request
		 * following the SETUP may be acknowledged */
		Usb_enable_stall_handshake();
		Usb_ack_receive_setup();
		return;
	}
	
	/* Send a ZLP for STATUS phase */	
	Usb_send_control_in();
	
	usb_user_endpoint_init(usb_configuration_nb);
	Usb_set_configuration_action();
}


/*! \brief Function to get the USB Descriptor
 *
 *  This function manages the GET DESCRIPTOR request. The device descriptor,
 *  the configuration descriptor and the device qualifier are supported. All
 *  other descriptors must be supported by the usb_user_get_descriptor
 *  function.
 *  Only 1 configuration is supported.
 */
void usb_get_descriptor(void)
{
	uint16_t  wLength         ;
	uint8_t  descriptor_type ;
	uint8_t  string_type     ;
	uint8_t  dummy;
	uint8_t  nb_byte;
	
	zlp             = false;                  /* no zero length packet */
	string_type     = Usb_read_byte();        /* read LSB of wValue    */
	descriptor_type = Usb_read_byte();        /* read MSB of wValue    */
	
	switch (descriptor_type)
	{
	    case DEVICE_DESCRIPTOR:
		data_to_transfer = Usb_get_dev_desc_length();
		pbuffer          = Usb_get_dev_desc_pointer();
		break;
	    case CONFIGURATION_DESCRIPTOR:
		data_to_transfer = Usb_get_conf_desc_length();
		pbuffer          = Usb_get_conf_desc_pointer();
		break;
	    default:
		if( usb_user_get_descriptor(descriptor_type, string_type)==false )
		{
			Usb_enable_stall_handshake();
			Usb_ack_receive_setup();
			return;
		}
		break;
	}
	
	/* Read out dummy parameters not needed. */
	dummy = Usb_read_byte();
	dummy = Usb_read_byte();
	
	/* Read out length of data. */
	LSB(wLength) = Usb_read_byte();
	MSB(wLength) = Usb_read_byte();
	Usb_ack_receive_setup();
	
	if (wLength > data_to_transfer)
	{
		if ((data_to_transfer % EP_CONTROL_LENGTH) == 0){
			zlp = true;
		}else{
			zlp = false;
		}
	}else{
		/* Send only requested number of data */
		data_to_transfer = (uint8_t)wLength;
	}
	
	while((data_to_transfer != 0) && (!Is_usb_receive_out())){

		while(!Is_usb_read_control_enabled());
		
		nb_byte=0;
		
		/* Send data until necessary */
		while(data_to_transfer != 0){
			if(nb_byte++==EP_CONTROL_LENGTH){
				break;
			}
			Usb_write_byte(*pbuffer++);
			data_to_transfer--;
		}
		Usb_send_control_in();
	}
	
	Usb_send_control_in();
	
	/* Abort from Host */
	if(Is_usb_receive_out()){
		Usb_ack_receive_out();
		return;
	}
	
	if(zlp == true){
		while(!Is_usb_read_control_enabled());
		Usb_send_control_in();
	}
	
	
	while(!Is_usb_receive_out());
	Usb_ack_receive_out();
}



/*! \brief This function manages the GET CONFIGURATION request. The current
 *         configuration number is returned.
 */
void usb_get_configuration(void)
{
	Usb_ack_receive_setup();
	
	Usb_write_byte(usb_configuration_nb);
	Usb_ack_in_ready();
	
	while( !Is_usb_receive_out() );
	Usb_ack_receive_out();
}


/*! \brief This function manages the GET STATUS request. The device, interface or
 *         endpoint status is returned.
 */
void usb_get_status(void)
{
	uint8_t wIndex;
	uint8_t dummy;
	
	/* Read out dummy parameters not needed. */
	dummy    = Usb_read_byte();
	dummy    = Usb_read_byte();
	
	wIndex = Usb_read_byte();
	
	switch(bmRequestType){
	    case REQUEST_DEVICE_STATUS:
		Usb_ack_receive_setup();
		Usb_write_byte(DEVICE_STATUS);
		break;
		
	    case REQUEST_INTERFACE_STATUS:
		Usb_ack_receive_setup();
		Usb_write_byte(INTERFACE_STATUS);
		break;
		
	    case REQUEST_ENDPOINT_STATUS:
		Usb_ack_receive_setup();
		wIndex = wIndex & MSK_EP_DIR;
		Usb_write_byte(endpoint_status[wIndex]);
		break;
		
	    default:
		Usb_enable_stall_handshake();
		Usb_ack_receive_setup();
		return;
	}
	
	Usb_write_byte(0x00);
	Usb_send_control_in();
	
	while( !Is_usb_receive_out() );
	Usb_ack_receive_out();
}


/*! \brief This function manages the SET FEATURE request. The USB test modes are
 *         supported by this function.
 */
void usb_set_feature(void)
{
	uint8_t wValue;
	uint8_t wIndex;
	uint8_t dummy;
	
	if (bmRequestType == INTERFACE_TYPE){
		/* Keep that order (set StallRq/clear RxSetup) or a OUT request
		 * following the SETUP may be acknowledged */
		Usb_enable_stall_handshake();
		Usb_ack_receive_setup();
	}else if (bmRequestType == ENDPOINT_TYPE){
		wValue = Usb_read_byte();
		dummy  = Usb_read_byte();
		
		if (wValue == FEATURE_ENDPOINT_HALT){
			wIndex = (Usb_read_byte() & MSK_EP_DIR);
			
			if (wIndex == EP_CONTROL){
				Usb_enable_stall_handshake();
				Usb_ack_receive_setup();
				return;
			}
			
			Usb_select_endpoint(wIndex);
			if(Is_usb_endpoint_enabled()){
				Usb_enable_stall_handshake();
				Usb_select_endpoint(EP_CONTROL);
				endpoint_status[wIndex] = 0x01;
				Usb_ack_receive_setup();
				Usb_send_control_in();
			}else{
				Usb_enable_stall_handshake();
				Usb_ack_receive_setup();
			}
		}else{
			Usb_enable_stall_handshake();
			Usb_ack_receive_setup();
		}
	}
}



/*! \brief This function manages the SET FEATURE request.
 */
void usb_clear_feature(void)
{
	uint8_t wValue;
	uint8_t wIndex;
	uint8_t dummy;
	
	if (bmRequestType == ZERO_TYPE){
		/* Keep that order (set StallRq/clear RxSetup) or a OUT request
		 * following the SETUP may be acknowledged */
		Usb_enable_stall_handshake();
		Usb_ack_receive_setup();
		return;
	}else if (bmRequestType == INTERFACE_TYPE){
		/* Keep that order (set StallRq/clear RxSetup) or a OUT request
		 * following the SETUP may be acknowledged */
		Usb_enable_stall_handshake();
		Usb_ack_receive_setup();
		return;
	}else if (bmRequestType == ENDPOINT_TYPE){
		wValue = Usb_read_byte();
		dummy  = Usb_read_byte();
		
		if (wValue == FEATURE_ENDPOINT_HALT){
			wIndex = (Usb_read_byte() & MSK_EP_DIR);
			
			Usb_select_endpoint(wIndex);
			if(Is_usb_endpoint_enabled()){
				if(wIndex != EP_CONTROL){
					Usb_disable_stall_handshake();
					Usb_reset_endpoint(wIndex);
					Usb_reset_data_toggle();
				}
				Usb_select_endpoint(EP_CONTROL);
				endpoint_status[wIndex] = 0x00;
				Usb_ack_receive_setup();
				Usb_send_control_in();
			}else{
				Usb_enable_stall_handshake();
				Usb_ack_receive_setup();
				return;
			}
		}else{
			Usb_enable_stall_handshake();
			Usb_ack_receive_setup();
			return;
		}
	}
}



/*! \brief This function manages the GET_INTERFACE request.
 */
void usb_get_interface (void)
{
	Usb_enable_stall_handshake();
	Usb_ack_receive_setup();
}


/*! \brief This function manages the SET_INTERFACE request.
 */
void usb_set_interface (void)
{
	Usb_ack_receive_setup();
	Usb_send_control_in();
	while(!Is_usb_in_ready());
}
