/* This file has been prepared for Doxygen automatic documentation generation.*/
/*! \file *********************************************************************
 *
 * \brief This file manages the USB task device.
 *
 *  This module also contains the general USB interrupt subroutine.
 *  This subroutine is used to detect asynchronous USB events.
 *
 * \par Application note:
 *      AVR1907: Xplain Evaluation Board
 *
 * \par Documentation
 *      For comprehensive code documentation, supported compilers, compiler
 *      settings and supported devices see readme.html
 *
 * \author
 *      Atmel Corporation: http://www.atmel.com \n
 *      Support email: avr@atmel.com
 *
 * $Revision: 3123 $
 * $Date: 2010-01-13 14:35:03 +0100 (on, 13 jan 2010) $  \n
 *
 * Copyright (c) 2010, Atmel Corporation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. The name of ATMEL may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ATMEL "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE EXPRESSLY AND
 * SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *****************************************************************************/

#include "config.h"
#include "conf_usb.h"
#include "usb_task.h"
#include "usb_drv.h"
#include "usb_descriptors.h"
#include "usb_device_task.h"



volatile uint16_t g_usb_event=0;
extern uint8_t usb_connected;
extern uint8_t usb_configuration_nb;



/*! \brief This function initializes the USB proces.
 *
 *  Depending on the mode supported (HOST/DEVICE/DUAL_ROLE) the function
 *  calls the coresponding usb mode initialization function
 */
void usb_task_init(void)
{
	Usb_force_device_mode();
	usb_device_task_init();
}


/*! \brief Entry point of the USB mamnagement
 *
 *  Depending on the USB mode supported (HOST/DEVICE/DUAL_ROLE) the function
 *  calls the coresponding usb management function.
 */
void usb_task(void)
{
	usb_device_task();
	
}


/*! \brief USB interrupt subroutine
 *
 *  This function is called each time a USB interrupt occurs.
 *  The following USB DEVICE events are taken in charge:
 *  - VBus On / Off
 *  - Start Of Frame
 *  - Suspend
 *  - Wake-Up
 *  - Resume
 *  - Reset
 *  - Start of frame
 *
 *
 *  For each event, the user can launch an action by completing
 *  the associate define (See conf_usb.h file to add action upon events)
 *
 *  \note Only interrupts events that are enabled are processed
 */
ISR(USB_GEN_vect)
{
	/* VBUS state detection */
	if (Is_usb_vbus_transition() && Is_usb_vbus_interrupt_enabled()){
		Usb_ack_vbus_transition();
		if (Is_usb_vbus_high()){
			usb_connected = true;
			Usb_vbus_on_action();
			Usb_send_event(EVT_USB_POWERED);
			Usb_enable_reset_interrupt();
			usb_start_device();
			Usb_attach();
		}else{
			Usb_vbus_off_action();
			usb_connected = false;
			Usb_send_event(EVT_USB_UNPOWERED);
		}
	}
	/* Device start of frame received  */
	if (Is_usb_sof() && Is_sof_interrupt_enabled()){
		Usb_ack_sof();
		Usb_sof_action();
	}
	/* Device Suspend event (no more USB activity detected)  */
	if (Is_usb_suspend() && Is_suspend_interrupt_enabled()){
		Usb_ack_suspend();
		Usb_enable_wake_up_interrupt();
		Usb_ack_wake_up();
		Usb_freeze_clock();
		Usb_send_event(EVT_USB_SUSPEND);
		Usb_suspend_action();
	}
	/* Wake up event (USB activity detected): Used to resume  */
	if (Is_usb_wake_up() && Is_swake_up_interrupt_enabled()){
		Usb_unfreeze_clock();
		Usb_ack_wake_up();
		Usb_disable_wake_up_interrupt();
		Usb_wake_up_action();
		Usb_send_event(EVT_USB_WAKE_UP);
	}
	
	/* Resume state bus detection  */
	if (Is_usb_resume() && Is_resume_interrupt_enabled()){
		Usb_disable_wake_up_interrupt();
		Usb_ack_resume();
		Usb_disable_resume_interrupt();
		Usb_resume_action();
		Usb_send_event(EVT_USB_RESUME);
	}
	
	/* USB bus reset detection  */
	if (Is_usb_reset()&& Is_reset_interrupt_enabled()){
		Usb_ack_reset();
		usb_init_device();
		Usb_reset_action();
		Usb_send_event(EVT_USB_RESET);
	}
	
}

/*! \brief Suspend action management
 *
 *  Use this function to handle the suspend command.
 */
extern void suspend_action(void)
{
	sei();
	//Enter_sleep(POWER_DOWN_MODE);
}
